# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_CREATE_SPHERE_METADATA = Metadata(
    id="bed788743061812c1852a66ee0671bb89ebb21bd.workbench",
    name="surface-create-sphere",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceCreateSphereParamsDictNoTag = typing.TypedDict('_SurfaceCreateSphereParamsDictNoTag', {
    "num-vertices": int,
    "sphere-out": str,
})
SurfaceCreateSphereParamsDictTagged = typing.TypedDict('SurfaceCreateSphereParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-create-sphere"],
    "num-vertices": int,
    "sphere-out": str,
})
SurfaceCreateSphereParamsDict = _SurfaceCreateSphereParamsDictNoTag | SurfaceCreateSphereParamsDictTagged


class SurfaceCreateSphereOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceCreateSphereParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    sphere_out: OutputPathType
    """the output sphere"""


def surface_create_sphere_params(
    num_vertices: int,
    sphere_out: str,
) -> SurfaceCreateSphereParamsDictTagged:
    """
    Build parameters.
    
    Args:
        num_vertices: desired number of vertices.
        sphere_out: the output sphere.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-create-sphere",
        "num-vertices": num_vertices,
        "sphere-out": sphere_out,
    }
    return params


def surface_create_sphere_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceCreateSphereParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("num-vertices", None) is None:
        raise StyxValidationError("`num-vertices` must not be None")
    if not isinstance(params["num-vertices"], int):
        raise StyxValidationError(f'`num-vertices` has the wrong type: Received `{type(params.get("num-vertices", None))}` expected `int`')
    if params.get("sphere-out", None) is None:
        raise StyxValidationError("`sphere-out` must not be None")
    if not isinstance(params["sphere-out"], str):
        raise StyxValidationError(f'`sphere-out` has the wrong type: Received `{type(params.get("sphere-out", None))}` expected `str`')


def surface_create_sphere_cargs(
    params: SurfaceCreateSphereParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-create-sphere"
    ])
    cargs.append(str(params.get("num-vertices", None)))
    cargs.append(params.get("sphere-out", None))
    return cargs


def surface_create_sphere_outputs(
    params: SurfaceCreateSphereParamsDict,
    execution: Execution,
) -> SurfaceCreateSphereOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceCreateSphereOutputs(
        root=execution.output_file("."),
        sphere_out=execution.output_file(params.get("sphere-out", None)),
    )
    return ret


def surface_create_sphere_execute(
    params: SurfaceCreateSphereParamsDict,
    runner: Runner | None = None,
) -> SurfaceCreateSphereOutputs:
    """
    GENERATE A SPHERE WITH CONSISTENT VERTEX AREAS.
    
    Generates a sphere by regularly dividing the triangles of an icosahedron, to
    come as close to the desired number of vertices as possible, and modifying
    it to have very similar vertex areas for all vertices. To generate a pair of
    vertex-matched left and right spheres, use this command, then
    -surface-flip-lr to generate the other sphere, then -set-structure on each.
    For example:
    
    $ wb_command -surface-create-sphere 6000 Sphere.6k.R.surf.gii
    $ wb_command -surface-flip-lr Sphere.6k.R.surf.gii Sphere.6k.L.surf.gii
    $ wb_command -set-structure Sphere.6k.R.surf.gii CORTEX_RIGHT
    $ wb_command -set-structure Sphere.6k.L.surf.gii CORTEX_LEFT.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceCreateSphereOutputs`).
    """
    surface_create_sphere_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_CREATE_SPHERE_METADATA)
    params = execution.params(params)
    cargs = surface_create_sphere_cargs(params, execution)
    ret = surface_create_sphere_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_create_sphere(
    num_vertices: int,
    sphere_out: str,
    runner: Runner | None = None,
) -> SurfaceCreateSphereOutputs:
    """
    GENERATE A SPHERE WITH CONSISTENT VERTEX AREAS.
    
    Generates a sphere by regularly dividing the triangles of an icosahedron, to
    come as close to the desired number of vertices as possible, and modifying
    it to have very similar vertex areas for all vertices. To generate a pair of
    vertex-matched left and right spheres, use this command, then
    -surface-flip-lr to generate the other sphere, then -set-structure on each.
    For example:
    
    $ wb_command -surface-create-sphere 6000 Sphere.6k.R.surf.gii
    $ wb_command -surface-flip-lr Sphere.6k.R.surf.gii Sphere.6k.L.surf.gii
    $ wb_command -set-structure Sphere.6k.R.surf.gii CORTEX_RIGHT
    $ wb_command -set-structure Sphere.6k.L.surf.gii CORTEX_LEFT.
    
    Args:
        num_vertices: desired number of vertices.
        sphere_out: the output sphere.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceCreateSphereOutputs`).
    """
    params = surface_create_sphere_params(
        num_vertices=num_vertices,
        sphere_out=sphere_out,
    )
    return surface_create_sphere_execute(params, runner)


__all__ = [
    "SURFACE_CREATE_SPHERE_METADATA",
    "SurfaceCreateSphereOutputs",
    "SurfaceCreateSphereParamsDict",
    "SurfaceCreateSphereParamsDictTagged",
    "surface_create_sphere",
    "surface_create_sphere_execute",
    "surface_create_sphere_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_GEODESIC_DISTANCE_SPARSE_TEXT_METADATA = Metadata(
    id="cd601df7eb26b8adf13c26cf609d763a8d458dd0.workbench",
    name="surface-geodesic-distance-sparse-text",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceGeodesicDistanceSparseTextParamsDictNoTag = typing.TypedDict('_SurfaceGeodesicDistanceSparseTextParamsDictNoTag', {
    "surface": InputPathType,
    "limit": float,
    "text-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
    "naive": bool,
})
SurfaceGeodesicDistanceSparseTextParamsDictTagged = typing.TypedDict('SurfaceGeodesicDistanceSparseTextParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-geodesic-distance-sparse-text"],
    "surface": InputPathType,
    "limit": float,
    "text-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
    "naive": bool,
})
SurfaceGeodesicDistanceSparseTextParamsDict = _SurfaceGeodesicDistanceSparseTextParamsDictNoTag | SurfaceGeodesicDistanceSparseTextParamsDictTagged


class SurfaceGeodesicDistanceSparseTextOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceGeodesicDistanceSparseTextParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def surface_geodesic_distance_sparse_text_params(
    surface: InputPathType,
    limit: float,
    text_out: str,
    area_metric: InputPathType | None = None,
    naive: bool = False,
) -> SurfaceGeodesicDistanceSparseTextParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to compute on.
        limit: the geodesic distance limit in mm.
        text_out: output - the output text file.
        area_metric: vertex areas to use to correct the distances on a\
            group-average surface\
            \
            the corrected vertex areas, as a metric.
        naive: use only neighbors, don't crawl triangles (not recommended).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-geodesic-distance-sparse-text",
        "surface": surface,
        "limit": limit,
        "text-out": text_out,
        "naive": naive,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def surface_geodesic_distance_sparse_text_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceGeodesicDistanceSparseTextParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("limit", None) is None:
        raise StyxValidationError("`limit` must not be None")
    if not isinstance(params["limit"], (float, int)):
        raise StyxValidationError(f'`limit` has the wrong type: Received `{type(params.get("limit", None))}` expected `float`')
    if params.get("text-out", None) is None:
        raise StyxValidationError("`text-out` must not be None")
    if not isinstance(params["text-out"], str):
        raise StyxValidationError(f'`text-out` has the wrong type: Received `{type(params.get("text-out", None))}` expected `str`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')
    if params.get("naive", False) is None:
        raise StyxValidationError("`naive` must not be None")
    if not isinstance(params["naive"], bool):
        raise StyxValidationError(f'`naive` has the wrong type: Received `{type(params.get("naive", False))}` expected `bool`')


def surface_geodesic_distance_sparse_text_cargs(
    params: SurfaceGeodesicDistanceSparseTextParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-geodesic-distance-sparse-text"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(str(params.get("limit", None)))
    cargs.append(params.get("text-out", None))
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    if params.get("naive", False):
        cargs.append("-naive")
    return cargs


def surface_geodesic_distance_sparse_text_outputs(
    params: SurfaceGeodesicDistanceSparseTextParamsDict,
    execution: Execution,
) -> SurfaceGeodesicDistanceSparseTextOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceGeodesicDistanceSparseTextOutputs(
        root=execution.output_file("."),
    )
    return ret


def surface_geodesic_distance_sparse_text_execute(
    params: SurfaceGeodesicDistanceSparseTextParamsDict,
    runner: Runner | None = None,
) -> SurfaceGeodesicDistanceSparseTextOutputs:
    """
    OUTPUT GEODESIC DISTANCES AS TEXT.
    
    Similar to -geodesic-distance-all-to-all, but outputs in a text format that
    only lists the in-range vertices. Each line in the output text file is a
    list of pairs, like '<vertex>,<distance>,<vertex>,<distance>,...' in csv
    format.
    
    The -corrected-areas option should be used when the input is a group average
    surface - group average surfaces have significantly less surface area than
    individual surfaces do, and therefore distances measured on them would be
    smaller than measuring them on individual surfaces. In this case, the input
    to this option should be a group average of the output of
    -surface-vertex-areas for each subject.
    
    If -naive is not specified, the algorithm uses not just immediate neighbors,
    but also neighbors derived from crawling across pairs of triangles that
    share an edge.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceGeodesicDistanceSparseTextOutputs`).
    """
    surface_geodesic_distance_sparse_text_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_GEODESIC_DISTANCE_SPARSE_TEXT_METADATA)
    params = execution.params(params)
    cargs = surface_geodesic_distance_sparse_text_cargs(params, execution)
    ret = surface_geodesic_distance_sparse_text_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_geodesic_distance_sparse_text(
    surface: InputPathType,
    limit: float,
    text_out: str,
    area_metric: InputPathType | None = None,
    naive: bool = False,
    runner: Runner | None = None,
) -> SurfaceGeodesicDistanceSparseTextOutputs:
    """
    OUTPUT GEODESIC DISTANCES AS TEXT.
    
    Similar to -geodesic-distance-all-to-all, but outputs in a text format that
    only lists the in-range vertices. Each line in the output text file is a
    list of pairs, like '<vertex>,<distance>,<vertex>,<distance>,...' in csv
    format.
    
    The -corrected-areas option should be used when the input is a group average
    surface - group average surfaces have significantly less surface area than
    individual surfaces do, and therefore distances measured on them would be
    smaller than measuring them on individual surfaces. In this case, the input
    to this option should be a group average of the output of
    -surface-vertex-areas for each subject.
    
    If -naive is not specified, the algorithm uses not just immediate neighbors,
    but also neighbors derived from crawling across pairs of triangles that
    share an edge.
    
    Args:
        surface: the surface to compute on.
        limit: the geodesic distance limit in mm.
        text_out: output - the output text file.
        area_metric: vertex areas to use to correct the distances on a\
            group-average surface\
            \
            the corrected vertex areas, as a metric.
        naive: use only neighbors, don't crawl triangles (not recommended).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceGeodesicDistanceSparseTextOutputs`).
    """
    params = surface_geodesic_distance_sparse_text_params(
        surface=surface,
        limit=limit,
        text_out=text_out,
        area_metric=area_metric,
        naive=naive,
    )
    return surface_geodesic_distance_sparse_text_execute(params, runner)


__all__ = [
    "SURFACE_GEODESIC_DISTANCE_SPARSE_TEXT_METADATA",
    "SurfaceGeodesicDistanceSparseTextOutputs",
    "SurfaceGeodesicDistanceSparseTextParamsDict",
    "SurfaceGeodesicDistanceSparseTextParamsDictTagged",
    "surface_geodesic_distance_sparse_text",
    "surface_geodesic_distance_sparse_text_execute",
    "surface_geodesic_distance_sparse_text_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_EXPORT_DENSE_MAPPING_METADATA = Metadata(
    id="caf8dad8ae4809da9a7eb9ca3301a06449390ae2.workbench",
    name="cifti-export-dense-mapping",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiExportDenseMappingVolumeAllParamsDictNoTag = typing.TypedDict('_CiftiExportDenseMappingVolumeAllParamsDictNoTag', {
    "text-out": str,
    "structure": bool,
    "no-cifti-index": bool,
})
CiftiExportDenseMappingVolumeAllParamsDictTagged = typing.TypedDict('CiftiExportDenseMappingVolumeAllParamsDictTagged', {
    "@type": typing.Literal["volume-all"],
    "text-out": str,
    "structure": bool,
    "no-cifti-index": bool,
})
CiftiExportDenseMappingVolumeAllParamsDict = _CiftiExportDenseMappingVolumeAllParamsDictNoTag | CiftiExportDenseMappingVolumeAllParamsDictTagged


_CiftiExportDenseMappingSurfaceParamsDictNoTag = typing.TypedDict('_CiftiExportDenseMappingSurfaceParamsDictNoTag', {
    "structure": str,
    "text-out": str,
    "no-cifti-index": bool,
})
CiftiExportDenseMappingSurfaceParamsDictTagged = typing.TypedDict('CiftiExportDenseMappingSurfaceParamsDictTagged', {
    "@type": typing.Literal["surface"],
    "structure": str,
    "text-out": str,
    "no-cifti-index": bool,
})
CiftiExportDenseMappingSurfaceParamsDict = _CiftiExportDenseMappingSurfaceParamsDictNoTag | CiftiExportDenseMappingSurfaceParamsDictTagged


_CiftiExportDenseMappingVolumeParamsDictNoTag = typing.TypedDict('_CiftiExportDenseMappingVolumeParamsDictNoTag', {
    "structure": str,
    "text-out": str,
    "no-cifti-index": bool,
})
CiftiExportDenseMappingVolumeParamsDictTagged = typing.TypedDict('CiftiExportDenseMappingVolumeParamsDictTagged', {
    "@type": typing.Literal["volume"],
    "structure": str,
    "text-out": str,
    "no-cifti-index": bool,
})
CiftiExportDenseMappingVolumeParamsDict = _CiftiExportDenseMappingVolumeParamsDictNoTag | CiftiExportDenseMappingVolumeParamsDictTagged


_CiftiExportDenseMappingParamsDictNoTag = typing.TypedDict('_CiftiExportDenseMappingParamsDictNoTag', {
    "cifti": InputPathType,
    "direction": str,
    "volume-all": typing.NotRequired[CiftiExportDenseMappingVolumeAllParamsDict | None],
    "surface": typing.NotRequired[list[CiftiExportDenseMappingSurfaceParamsDict] | None],
    "volume": typing.NotRequired[list[CiftiExportDenseMappingVolumeParamsDict] | None],
})
CiftiExportDenseMappingParamsDictTagged = typing.TypedDict('CiftiExportDenseMappingParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-export-dense-mapping"],
    "cifti": InputPathType,
    "direction": str,
    "volume-all": typing.NotRequired[CiftiExportDenseMappingVolumeAllParamsDict | None],
    "surface": typing.NotRequired[list[CiftiExportDenseMappingSurfaceParamsDict] | None],
    "volume": typing.NotRequired[list[CiftiExportDenseMappingVolumeParamsDict] | None],
})
CiftiExportDenseMappingParamsDict = _CiftiExportDenseMappingParamsDictNoTag | CiftiExportDenseMappingParamsDictTagged


def cifti_export_dense_mapping_volume_all(
    text_out: str,
    structure: bool = False,
    no_cifti_index: bool = False,
) -> CiftiExportDenseMappingVolumeAllParamsDictTagged:
    """
    Build parameters.
    
    Args:
        text_out: output - the output text file.
        structure: write the structure each voxel belongs to in the output file.
        no_cifti_index: don't write the cifti index in the output file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume-all",
        "text-out": text_out,
        "structure": structure,
        "no-cifti-index": no_cifti_index,
    }
    return params


def cifti_export_dense_mapping_volume_all_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiExportDenseMappingVolumeAllParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("text-out", None) is None:
        raise StyxValidationError("`text-out` must not be None")
    if not isinstance(params["text-out"], str):
        raise StyxValidationError(f'`text-out` has the wrong type: Received `{type(params.get("text-out", None))}` expected `str`')
    if params.get("structure", False) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], bool):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", False))}` expected `bool`')
    if params.get("no-cifti-index", False) is None:
        raise StyxValidationError("`no-cifti-index` must not be None")
    if not isinstance(params["no-cifti-index"], bool):
        raise StyxValidationError(f'`no-cifti-index` has the wrong type: Received `{type(params.get("no-cifti-index", False))}` expected `bool`')


def cifti_export_dense_mapping_volume_all_cargs(
    params: CiftiExportDenseMappingVolumeAllParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume-all",
        params.get("text-out", None)
    ])
    if params.get("structure", False):
        cargs.append("-structure")
    if params.get("no-cifti-index", False):
        cargs.append("-no-cifti-index")
    return cargs


def cifti_export_dense_mapping_surface(
    structure: str,
    text_out: str,
    no_cifti_index: bool = False,
) -> CiftiExportDenseMappingSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure to output.
        text_out: output - the output text file.
        no_cifti_index: don't write the cifti index in the output file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "surface",
        "structure": structure,
        "text-out": text_out,
        "no-cifti-index": no_cifti_index,
    }
    return params


def cifti_export_dense_mapping_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiExportDenseMappingSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("text-out", None) is None:
        raise StyxValidationError("`text-out` must not be None")
    if not isinstance(params["text-out"], str):
        raise StyxValidationError(f'`text-out` has the wrong type: Received `{type(params.get("text-out", None))}` expected `str`')
    if params.get("no-cifti-index", False) is None:
        raise StyxValidationError("`no-cifti-index` must not be None")
    if not isinstance(params["no-cifti-index"], bool):
        raise StyxValidationError(f'`no-cifti-index` has the wrong type: Received `{type(params.get("no-cifti-index", False))}` expected `bool`')


def cifti_export_dense_mapping_surface_cargs(
    params: CiftiExportDenseMappingSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-surface",
        params.get("structure", None),
        params.get("text-out", None)
    ])
    if params.get("no-cifti-index", False):
        cargs.append("-no-cifti-index")
    return cargs


def cifti_export_dense_mapping_volume(
    structure: str,
    text_out: str,
    no_cifti_index: bool = False,
) -> CiftiExportDenseMappingVolumeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure to output.
        text_out: output - the output text file.
        no_cifti_index: don't write the cifti index in the output file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume",
        "structure": structure,
        "text-out": text_out,
        "no-cifti-index": no_cifti_index,
    }
    return params


def cifti_export_dense_mapping_volume_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiExportDenseMappingVolumeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("text-out", None) is None:
        raise StyxValidationError("`text-out` must not be None")
    if not isinstance(params["text-out"], str):
        raise StyxValidationError(f'`text-out` has the wrong type: Received `{type(params.get("text-out", None))}` expected `str`')
    if params.get("no-cifti-index", False) is None:
        raise StyxValidationError("`no-cifti-index` must not be None")
    if not isinstance(params["no-cifti-index"], bool):
        raise StyxValidationError(f'`no-cifti-index` has the wrong type: Received `{type(params.get("no-cifti-index", False))}` expected `bool`')


def cifti_export_dense_mapping_volume_cargs(
    params: CiftiExportDenseMappingVolumeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume",
        params.get("structure", None),
        params.get("text-out", None)
    ])
    if params.get("no-cifti-index", False):
        cargs.append("-no-cifti-index")
    return cargs


class CiftiExportDenseMappingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiExportDenseMappingParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def cifti_export_dense_mapping_params(
    cifti: InputPathType,
    direction: str,
    volume_all: CiftiExportDenseMappingVolumeAllParamsDict | None = None,
    surface: list[CiftiExportDenseMappingSurfaceParamsDict] | None = None,
    volume: list[CiftiExportDenseMappingVolumeParamsDict] | None = None,
) -> CiftiExportDenseMappingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti: the cifti file.
        direction: which direction to export the mapping from, ROW or COLUMN.
        volume_all: export the the mapping of all voxels.
        surface: export the the mapping of one surface structure.
        volume: export the the mapping of one volume structure.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-export-dense-mapping",
        "cifti": cifti,
        "direction": direction,
    }
    if volume_all is not None:
        params["volume-all"] = volume_all
    if surface is not None:
        params["surface"] = surface
    if volume is not None:
        params["volume"] = volume
    return params


def cifti_export_dense_mapping_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiExportDenseMappingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti", None) is None:
        raise StyxValidationError("`cifti` must not be None")
    if not isinstance(params["cifti"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `InputPathType`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("volume-all", None) is not None:
        cifti_export_dense_mapping_volume_all_validate(params["volume-all"])
    if params.get("surface", None) is not None:
        if not isinstance(params["surface"], list):
            raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `list[CiftiExportDenseMappingSurfaceParamsDict] | None`')
        for e in params["surface"]:
            cifti_export_dense_mapping_surface_validate(e)
    if params.get("volume", None) is not None:
        if not isinstance(params["volume"], list):
            raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `list[CiftiExportDenseMappingVolumeParamsDict] | None`')
        for e in params["volume"]:
            cifti_export_dense_mapping_volume_validate(e)


def cifti_export_dense_mapping_cargs(
    params: CiftiExportDenseMappingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-export-dense-mapping"
    ])
    cargs.append(execution.input_file(params.get("cifti", None)))
    cargs.append(params.get("direction", None))
    if params.get("volume-all", None) is not None or params.get("surface", None) is not None or params.get("volume", None) is not None:
        cargs.extend([
            *(cifti_export_dense_mapping_volume_all_cargs(params.get("volume-all", None), execution) if (params.get("volume-all", None) is not None) else []),
            *([a for c in [cifti_export_dense_mapping_surface_cargs(s, execution) for s in params.get("surface", None)] for a in c] if (params.get("surface", None) is not None) else []),
            *([a for c in [cifti_export_dense_mapping_volume_cargs(s, execution) for s in params.get("volume", None)] for a in c] if (params.get("volume", None) is not None) else [])
        ])
    return cargs


def cifti_export_dense_mapping_outputs(
    params: CiftiExportDenseMappingParamsDict,
    execution: Execution,
) -> CiftiExportDenseMappingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiExportDenseMappingOutputs(
        root=execution.output_file("."),
    )
    return ret


def cifti_export_dense_mapping_execute(
    params: CiftiExportDenseMappingParamsDict,
    runner: Runner | None = None,
) -> CiftiExportDenseMappingOutputs:
    """
    WRITE INDEX TO ELEMENT MAPPING AS TEXT.
    
    This command produces text files that describe the mapping from cifti
    indices to surface vertices or voxels. All indices are zero-based. The
    default format for -surface is lines of the form:
    
    <cifti-index> <vertex>
    
    The default format for -volume and -volume-all is lines of the form:
    
    <cifti-index> <i> <j> <k>
    
    For each <structure> argument, use one of the following strings:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiExportDenseMappingOutputs`).
    """
    cifti_export_dense_mapping_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_EXPORT_DENSE_MAPPING_METADATA)
    params = execution.params(params)
    cargs = cifti_export_dense_mapping_cargs(params, execution)
    ret = cifti_export_dense_mapping_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_export_dense_mapping(
    cifti: InputPathType,
    direction: str,
    volume_all: CiftiExportDenseMappingVolumeAllParamsDict | None = None,
    surface: list[CiftiExportDenseMappingSurfaceParamsDict] | None = None,
    volume: list[CiftiExportDenseMappingVolumeParamsDict] | None = None,
    runner: Runner | None = None,
) -> CiftiExportDenseMappingOutputs:
    """
    WRITE INDEX TO ELEMENT MAPPING AS TEXT.
    
    This command produces text files that describe the mapping from cifti
    indices to surface vertices or voxels. All indices are zero-based. The
    default format for -surface is lines of the form:
    
    <cifti-index> <vertex>
    
    The default format for -volume and -volume-all is lines of the form:
    
    <cifti-index> <i> <j> <k>
    
    For each <structure> argument, use one of the following strings:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        cifti: the cifti file.
        direction: which direction to export the mapping from, ROW or COLUMN.
        volume_all: export the the mapping of all voxels.
        surface: export the the mapping of one surface structure.
        volume: export the the mapping of one volume structure.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiExportDenseMappingOutputs`).
    """
    params = cifti_export_dense_mapping_params(
        cifti=cifti,
        direction=direction,
        volume_all=volume_all,
        surface=surface,
        volume=volume,
    )
    return cifti_export_dense_mapping_execute(params, runner)


__all__ = [
    "CIFTI_EXPORT_DENSE_MAPPING_METADATA",
    "CiftiExportDenseMappingOutputs",
    "CiftiExportDenseMappingParamsDict",
    "CiftiExportDenseMappingParamsDictTagged",
    "CiftiExportDenseMappingSurfaceParamsDict",
    "CiftiExportDenseMappingSurfaceParamsDictTagged",
    "CiftiExportDenseMappingVolumeAllParamsDict",
    "CiftiExportDenseMappingVolumeAllParamsDictTagged",
    "CiftiExportDenseMappingVolumeParamsDict",
    "CiftiExportDenseMappingVolumeParamsDictTagged",
    "cifti_export_dense_mapping",
    "cifti_export_dense_mapping_execute",
    "cifti_export_dense_mapping_params",
    "cifti_export_dense_mapping_surface",
    "cifti_export_dense_mapping_volume",
    "cifti_export_dense_mapping_volume_all",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_FLIP_NORMALS_METADATA = Metadata(
    id="bd50d144fdc0e871fae85f49e492aee6de124549.workbench",
    name="surface-flip-normals",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceFlipNormalsParamsDictNoTag = typing.TypedDict('_SurfaceFlipNormalsParamsDictNoTag', {
    "surface": InputPathType,
    "surface-out": str,
})
SurfaceFlipNormalsParamsDictTagged = typing.TypedDict('SurfaceFlipNormalsParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-flip-normals"],
    "surface": InputPathType,
    "surface-out": str,
})
SurfaceFlipNormalsParamsDict = _SurfaceFlipNormalsParamsDictNoTag | SurfaceFlipNormalsParamsDictTagged


class SurfaceFlipNormalsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceFlipNormalsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    surface_out: OutputPathType
    """the output surface"""


def surface_flip_normals_params(
    surface: InputPathType,
    surface_out: str,
) -> SurfaceFlipNormalsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to flip the normals of.
        surface_out: the output surface.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-flip-normals",
        "surface": surface,
        "surface-out": surface_out,
    }
    return params


def surface_flip_normals_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceFlipNormalsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("surface-out", None) is None:
        raise StyxValidationError("`surface-out` must not be None")
    if not isinstance(params["surface-out"], str):
        raise StyxValidationError(f'`surface-out` has the wrong type: Received `{type(params.get("surface-out", None))}` expected `str`')


def surface_flip_normals_cargs(
    params: SurfaceFlipNormalsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-flip-normals"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(params.get("surface-out", None))
    return cargs


def surface_flip_normals_outputs(
    params: SurfaceFlipNormalsParamsDict,
    execution: Execution,
) -> SurfaceFlipNormalsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceFlipNormalsOutputs(
        root=execution.output_file("."),
        surface_out=execution.output_file(params.get("surface-out", None)),
    )
    return ret


def surface_flip_normals_execute(
    params: SurfaceFlipNormalsParamsDict,
    runner: Runner | None = None,
) -> SurfaceFlipNormalsOutputs:
    """
    FLIP ALL TILES ON A SURFACE.
    
    Flips all triangles on a surface, resulting in surface normals being flipped
    the other direction (inward vs outward). If you transform a surface with an
    affine that has negative determinant, or a warpfield that similarly flips
    the surface, you may end up with a surface that has normals pointing
    inwards, which may have display problems. Using this command will solve that
    problem.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceFlipNormalsOutputs`).
    """
    surface_flip_normals_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_FLIP_NORMALS_METADATA)
    params = execution.params(params)
    cargs = surface_flip_normals_cargs(params, execution)
    ret = surface_flip_normals_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_flip_normals(
    surface: InputPathType,
    surface_out: str,
    runner: Runner | None = None,
) -> SurfaceFlipNormalsOutputs:
    """
    FLIP ALL TILES ON A SURFACE.
    
    Flips all triangles on a surface, resulting in surface normals being flipped
    the other direction (inward vs outward). If you transform a surface with an
    affine that has negative determinant, or a warpfield that similarly flips
    the surface, you may end up with a surface that has normals pointing
    inwards, which may have display problems. Using this command will solve that
    problem.
    
    Args:
        surface: the surface to flip the normals of.
        surface_out: the output surface.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceFlipNormalsOutputs`).
    """
    params = surface_flip_normals_params(
        surface=surface,
        surface_out=surface_out,
    )
    return surface_flip_normals_execute(params, runner)


__all__ = [
    "SURFACE_FLIP_NORMALS_METADATA",
    "SurfaceFlipNormalsOutputs",
    "SurfaceFlipNormalsParamsDict",
    "SurfaceFlipNormalsParamsDictTagged",
    "surface_flip_normals",
    "surface_flip_normals_execute",
    "surface_flip_normals_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CONVERT_FIBER_ORIENTATIONS_METADATA = Metadata(
    id="c03f58871879950a5d06809f88add9efe24fe896.workbench",
    name="convert-fiber-orientations",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_ConvertFiberOrientationsFiberParamsDictNoTag = typing.TypedDict('_ConvertFiberOrientationsFiberParamsDictNoTag', {
    "mean-f": InputPathType,
    "stdev-f": InputPathType,
    "theta": InputPathType,
    "phi": InputPathType,
    "psi": InputPathType,
    "ka": InputPathType,
    "kb": InputPathType,
})
ConvertFiberOrientationsFiberParamsDictTagged = typing.TypedDict('ConvertFiberOrientationsFiberParamsDictTagged', {
    "@type": typing.Literal["fiber"],
    "mean-f": InputPathType,
    "stdev-f": InputPathType,
    "theta": InputPathType,
    "phi": InputPathType,
    "psi": InputPathType,
    "ka": InputPathType,
    "kb": InputPathType,
})
ConvertFiberOrientationsFiberParamsDict = _ConvertFiberOrientationsFiberParamsDictNoTag | ConvertFiberOrientationsFiberParamsDictTagged


_ConvertFiberOrientationsParamsDictNoTag = typing.TypedDict('_ConvertFiberOrientationsParamsDictNoTag', {
    "label-volume": InputPathType,
    "fiber-out": str,
    "fiber": typing.NotRequired[list[ConvertFiberOrientationsFiberParamsDict] | None],
})
ConvertFiberOrientationsParamsDictTagged = typing.TypedDict('ConvertFiberOrientationsParamsDictTagged', {
    "@type": typing.Literal["workbench/convert-fiber-orientations"],
    "label-volume": InputPathType,
    "fiber-out": str,
    "fiber": typing.NotRequired[list[ConvertFiberOrientationsFiberParamsDict] | None],
})
ConvertFiberOrientationsParamsDict = _ConvertFiberOrientationsParamsDictNoTag | ConvertFiberOrientationsParamsDictTagged


def convert_fiber_orientations_fiber(
    mean_f: InputPathType,
    stdev_f: InputPathType,
    theta: InputPathType,
    phi: InputPathType,
    psi: InputPathType,
    ka: InputPathType,
    kb: InputPathType,
) -> ConvertFiberOrientationsFiberParamsDictTagged:
    """
    Build parameters.
    
    Args:
        mean_f: mean fiber strength.
        stdev_f: standard deviation of fiber strength.
        theta: theta angle.
        phi: phi angle.
        psi: psi angle.
        ka: ka bingham parameter.
        kb: kb bingham parameter.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fiber",
        "mean-f": mean_f,
        "stdev-f": stdev_f,
        "theta": theta,
        "phi": phi,
        "psi": psi,
        "ka": ka,
        "kb": kb,
    }
    return params


def convert_fiber_orientations_fiber_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertFiberOrientationsFiberParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("mean-f", None) is None:
        raise StyxValidationError("`mean-f` must not be None")
    if not isinstance(params["mean-f"], (pathlib.Path, str)):
        raise StyxValidationError(f'`mean-f` has the wrong type: Received `{type(params.get("mean-f", None))}` expected `InputPathType`')
    if params.get("stdev-f", None) is None:
        raise StyxValidationError("`stdev-f` must not be None")
    if not isinstance(params["stdev-f"], (pathlib.Path, str)):
        raise StyxValidationError(f'`stdev-f` has the wrong type: Received `{type(params.get("stdev-f", None))}` expected `InputPathType`')
    if params.get("theta", None) is None:
        raise StyxValidationError("`theta` must not be None")
    if not isinstance(params["theta"], (pathlib.Path, str)):
        raise StyxValidationError(f'`theta` has the wrong type: Received `{type(params.get("theta", None))}` expected `InputPathType`')
    if params.get("phi", None) is None:
        raise StyxValidationError("`phi` must not be None")
    if not isinstance(params["phi"], (pathlib.Path, str)):
        raise StyxValidationError(f'`phi` has the wrong type: Received `{type(params.get("phi", None))}` expected `InputPathType`')
    if params.get("psi", None) is None:
        raise StyxValidationError("`psi` must not be None")
    if not isinstance(params["psi"], (pathlib.Path, str)):
        raise StyxValidationError(f'`psi` has the wrong type: Received `{type(params.get("psi", None))}` expected `InputPathType`')
    if params.get("ka", None) is None:
        raise StyxValidationError("`ka` must not be None")
    if not isinstance(params["ka"], (pathlib.Path, str)):
        raise StyxValidationError(f'`ka` has the wrong type: Received `{type(params.get("ka", None))}` expected `InputPathType`')
    if params.get("kb", None) is None:
        raise StyxValidationError("`kb` must not be None")
    if not isinstance(params["kb"], (pathlib.Path, str)):
        raise StyxValidationError(f'`kb` has the wrong type: Received `{type(params.get("kb", None))}` expected `InputPathType`')


def convert_fiber_orientations_fiber_cargs(
    params: ConvertFiberOrientationsFiberParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-fiber",
        execution.input_file(params.get("mean-f", None)),
        execution.input_file(params.get("stdev-f", None)),
        execution.input_file(params.get("theta", None)),
        execution.input_file(params.get("phi", None)),
        execution.input_file(params.get("psi", None)),
        execution.input_file(params.get("ka", None)),
        execution.input_file(params.get("kb", None))
    ])
    return cargs


class ConvertFiberOrientationsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ConvertFiberOrientationsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    fiber_out: OutputPathType
    """the output fiber orientation file"""


def convert_fiber_orientations_params(
    label_volume: InputPathType,
    fiber_out: str,
    fiber: list[ConvertFiberOrientationsFiberParamsDict] | None = None,
) -> ConvertFiberOrientationsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_volume: volume of cifti structure labels.
        fiber_out: the output fiber orientation file.
        fiber: specify the parameter volumes for a fiber.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/convert-fiber-orientations",
        "label-volume": label_volume,
        "fiber-out": fiber_out,
    }
    if fiber is not None:
        params["fiber"] = fiber
    return params


def convert_fiber_orientations_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertFiberOrientationsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-volume", None) is None:
        raise StyxValidationError("`label-volume` must not be None")
    if not isinstance(params["label-volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-volume` has the wrong type: Received `{type(params.get("label-volume", None))}` expected `InputPathType`')
    if params.get("fiber-out", None) is None:
        raise StyxValidationError("`fiber-out` must not be None")
    if not isinstance(params["fiber-out"], str):
        raise StyxValidationError(f'`fiber-out` has the wrong type: Received `{type(params.get("fiber-out", None))}` expected `str`')
    if params.get("fiber", None) is not None:
        if not isinstance(params["fiber"], list):
            raise StyxValidationError(f'`fiber` has the wrong type: Received `{type(params.get("fiber", None))}` expected `list[ConvertFiberOrientationsFiberParamsDict] | None`')
        for e in params["fiber"]:
            convert_fiber_orientations_fiber_validate(e)


def convert_fiber_orientations_cargs(
    params: ConvertFiberOrientationsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-convert-fiber-orientations"
    ])
    cargs.append(execution.input_file(params.get("label-volume", None)))
    cargs.append(params.get("fiber-out", None))
    if params.get("fiber", None) is not None:
        cargs.extend([a for c in [convert_fiber_orientations_fiber_cargs(s, execution) for s in params.get("fiber", None)] for a in c])
    return cargs


def convert_fiber_orientations_outputs(
    params: ConvertFiberOrientationsParamsDict,
    execution: Execution,
) -> ConvertFiberOrientationsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ConvertFiberOrientationsOutputs(
        root=execution.output_file("."),
        fiber_out=execution.output_file(params.get("fiber-out", None)),
    )
    return ret


def convert_fiber_orientations_execute(
    params: ConvertFiberOrientationsParamsDict,
    runner: Runner | None = None,
) -> ConvertFiberOrientationsOutputs:
    """
    CONVERT BINGHAM PARAMETER VOLUMES TO FIBER ORIENTATION FILE.
    
    Takes precomputed bingham parameters from volume files and converts them to
    the format workbench uses for display. The <label-volume> argument must be a
    label volume, where the labels use these strings:
    
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertFiberOrientationsOutputs`).
    """
    convert_fiber_orientations_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVERT_FIBER_ORIENTATIONS_METADATA)
    params = execution.params(params)
    cargs = convert_fiber_orientations_cargs(params, execution)
    ret = convert_fiber_orientations_outputs(params, execution)
    execution.run(cargs)
    return ret


def convert_fiber_orientations(
    label_volume: InputPathType,
    fiber_out: str,
    fiber: list[ConvertFiberOrientationsFiberParamsDict] | None = None,
    runner: Runner | None = None,
) -> ConvertFiberOrientationsOutputs:
    """
    CONVERT BINGHAM PARAMETER VOLUMES TO FIBER ORIENTATION FILE.
    
    Takes precomputed bingham parameters from volume files and converts them to
    the format workbench uses for display. The <label-volume> argument must be a
    label volume, where the labels use these strings:
    
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        label_volume: volume of cifti structure labels.
        fiber_out: the output fiber orientation file.
        fiber: specify the parameter volumes for a fiber.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertFiberOrientationsOutputs`).
    """
    params = convert_fiber_orientations_params(
        label_volume=label_volume,
        fiber_out=fiber_out,
        fiber=fiber,
    )
    return convert_fiber_orientations_execute(params, runner)


__all__ = [
    "CONVERT_FIBER_ORIENTATIONS_METADATA",
    "ConvertFiberOrientationsFiberParamsDict",
    "ConvertFiberOrientationsFiberParamsDictTagged",
    "ConvertFiberOrientationsOutputs",
    "ConvertFiberOrientationsParamsDict",
    "ConvertFiberOrientationsParamsDictTagged",
    "convert_fiber_orientations",
    "convert_fiber_orientations_execute",
    "convert_fiber_orientations_fiber",
    "convert_fiber_orientations_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_WEDGE_VOLUME_METADATA = Metadata(
    id="836d42477cc472d35639defeb9265c3084b789a0.workbench",
    name="surface-wedge-volume",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceWedgeVolumeParamsDictNoTag = typing.TypedDict('_SurfaceWedgeVolumeParamsDictNoTag', {
    "inner-surface": InputPathType,
    "outer-surface": InputPathType,
    "metric": str,
})
SurfaceWedgeVolumeParamsDictTagged = typing.TypedDict('SurfaceWedgeVolumeParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-wedge-volume"],
    "inner-surface": InputPathType,
    "outer-surface": InputPathType,
    "metric": str,
})
SurfaceWedgeVolumeParamsDict = _SurfaceWedgeVolumeParamsDictNoTag | SurfaceWedgeVolumeParamsDictTagged


class SurfaceWedgeVolumeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceWedgeVolumeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric: OutputPathType
    """the output metric"""


def surface_wedge_volume_params(
    inner_surface: InputPathType,
    outer_surface: InputPathType,
    metric: str,
) -> SurfaceWedgeVolumeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        inner_surface: the inner surface.
        outer_surface: the outer surface.
        metric: the output metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-wedge-volume",
        "inner-surface": inner_surface,
        "outer-surface": outer_surface,
        "metric": metric,
    }
    return params


def surface_wedge_volume_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceWedgeVolumeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("inner-surface", None) is None:
        raise StyxValidationError("`inner-surface` must not be None")
    if not isinstance(params["inner-surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`inner-surface` has the wrong type: Received `{type(params.get("inner-surface", None))}` expected `InputPathType`')
    if params.get("outer-surface", None) is None:
        raise StyxValidationError("`outer-surface` must not be None")
    if not isinstance(params["outer-surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`outer-surface` has the wrong type: Received `{type(params.get("outer-surface", None))}` expected `InputPathType`')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], str):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `str`')


def surface_wedge_volume_cargs(
    params: SurfaceWedgeVolumeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-wedge-volume"
    ])
    cargs.append(execution.input_file(params.get("inner-surface", None)))
    cargs.append(execution.input_file(params.get("outer-surface", None)))
    cargs.append(params.get("metric", None))
    return cargs


def surface_wedge_volume_outputs(
    params: SurfaceWedgeVolumeParamsDict,
    execution: Execution,
) -> SurfaceWedgeVolumeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceWedgeVolumeOutputs(
        root=execution.output_file("."),
        metric=execution.output_file(params.get("metric", None)),
    )
    return ret


def surface_wedge_volume_execute(
    params: SurfaceWedgeVolumeParamsDict,
    runner: Runner | None = None,
) -> SurfaceWedgeVolumeOutputs:
    """
    MEASURE PER-VERTEX VOLUME BETWEEN SURFACES.
    
    Compute the volume of each vertex's area from one surface to another. The
    surfaces must have vertex correspondence, and have consistent triangle
    orientation.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceWedgeVolumeOutputs`).
    """
    surface_wedge_volume_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_WEDGE_VOLUME_METADATA)
    params = execution.params(params)
    cargs = surface_wedge_volume_cargs(params, execution)
    ret = surface_wedge_volume_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_wedge_volume(
    inner_surface: InputPathType,
    outer_surface: InputPathType,
    metric: str,
    runner: Runner | None = None,
) -> SurfaceWedgeVolumeOutputs:
    """
    MEASURE PER-VERTEX VOLUME BETWEEN SURFACES.
    
    Compute the volume of each vertex's area from one surface to another. The
    surfaces must have vertex correspondence, and have consistent triangle
    orientation.
    
    Args:
        inner_surface: the inner surface.
        outer_surface: the outer surface.
        metric: the output metric.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceWedgeVolumeOutputs`).
    """
    params = surface_wedge_volume_params(
        inner_surface=inner_surface,
        outer_surface=outer_surface,
        metric=metric,
    )
    return surface_wedge_volume_execute(params, runner)


__all__ = [
    "SURFACE_WEDGE_VOLUME_METADATA",
    "SurfaceWedgeVolumeOutputs",
    "SurfaceWedgeVolumeParamsDict",
    "SurfaceWedgeVolumeParamsDictTagged",
    "surface_wedge_volume",
    "surface_wedge_volume_execute",
    "surface_wedge_volume_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_LABEL_IMPORT_METADATA = Metadata(
    id="a6ac49efad7e33730c569a6154b9c1ecb184a663.workbench",
    name="cifti-label-import",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiLabelImportParamsDictNoTag = typing.TypedDict('_CiftiLabelImportParamsDictNoTag', {
    "input": InputPathType,
    "label-list-file": str,
    "output": str,
    "file": typing.NotRequired[str | None],
    "value": typing.NotRequired[int | None],
    "drop-unused-labels": bool,
    "discard-others": bool,
})
CiftiLabelImportParamsDictTagged = typing.TypedDict('CiftiLabelImportParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-label-import"],
    "input": InputPathType,
    "label-list-file": str,
    "output": str,
    "file": typing.NotRequired[str | None],
    "value": typing.NotRequired[int | None],
    "drop-unused-labels": bool,
    "discard-others": bool,
})
CiftiLabelImportParamsDict = _CiftiLabelImportParamsDictNoTag | CiftiLabelImportParamsDictTagged


class CiftiLabelImportOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiLabelImportParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output cifti label file"""


def cifti_label_import_params(
    input_: InputPathType,
    label_list_file: str,
    output: str,
    file: str | None = None,
    value: int | None = None,
    drop_unused_labels: bool = False,
    discard_others: bool = False,
) -> CiftiLabelImportParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input cifti file.
        label_list_file: text file containing the values and names for labels.
        output: the output cifti label file.
        file: read label name hierarchy from a json file\
            \
            the input json file.
        value: set the value that will be interpreted as unlabeled\
            \
            the numeric value for unlabeled (default 0).
        drop_unused_labels: remove any unused label values from the label table.
        discard_others: set any values not mentioned in the label list to the\
            ??? label.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-label-import",
        "input": input_,
        "label-list-file": label_list_file,
        "output": output,
        "drop-unused-labels": drop_unused_labels,
        "discard-others": discard_others,
    }
    if file is not None:
        params["file"] = file
    if value is not None:
        params["value"] = value
    return params


def cifti_label_import_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiLabelImportParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("label-list-file", None) is None:
        raise StyxValidationError("`label-list-file` must not be None")
    if not isinstance(params["label-list-file"], str):
        raise StyxValidationError(f'`label-list-file` has the wrong type: Received `{type(params.get("label-list-file", None))}` expected `str`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("file", None) is not None:
        if not isinstance(params["file"], str):
            raise StyxValidationError(f'`file` has the wrong type: Received `{type(params.get("file", None))}` expected `str | None`')
    if params.get("value", None) is not None:
        if not isinstance(params["value"], int):
            raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `int | None`')
    if params.get("drop-unused-labels", False) is None:
        raise StyxValidationError("`drop-unused-labels` must not be None")
    if not isinstance(params["drop-unused-labels"], bool):
        raise StyxValidationError(f'`drop-unused-labels` has the wrong type: Received `{type(params.get("drop-unused-labels", False))}` expected `bool`')
    if params.get("discard-others", False) is None:
        raise StyxValidationError("`discard-others` must not be None")
    if not isinstance(params["discard-others"], bool):
        raise StyxValidationError(f'`discard-others` has the wrong type: Received `{type(params.get("discard-others", False))}` expected `bool`')


def cifti_label_import_cargs(
    params: CiftiLabelImportParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-label-import"
    ])
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("label-list-file", None))
    cargs.append(params.get("output", None))
    if params.get("file", None) is not None:
        cargs.extend([
            "-hierarchy",
            params.get("file", None)
        ])
    if params.get("value", None) is not None:
        cargs.extend([
            "-unlabeled-value",
            str(params.get("value", None))
        ])
    if params.get("drop-unused-labels", False):
        cargs.append("-drop-unused-labels")
    if params.get("discard-others", False):
        cargs.append("-discard-others")
    return cargs


def cifti_label_import_outputs(
    params: CiftiLabelImportParamsDict,
    execution: Execution,
) -> CiftiLabelImportOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiLabelImportOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def cifti_label_import_execute(
    params: CiftiLabelImportParamsDict,
    runner: Runner | None = None,
) -> CiftiLabelImportOutputs:
    """
    MAKE A CIFTI LABEL FILE FROM A CIFTI FILE.
    
    Creates a cifti label file from a cifti file with label-like values. You may
    specify the empty string (use "") for <label-list-file>, which will be
    treated as if it is an empty file. The label list file must have the
    following format (2 lines per label):
    
    <labelname>
    <key> <red> <green> <blue> <alpha>
    ...
    
    Label names are specified on a separate line from their value and color, in
    order to let label names contain spaces. Whitespace is trimmed from both
    ends of the label name, but is kept if it is in the middle of a label. Do
    not specify the "unlabeled" key in the file, it is assumed that 0 means not
    labeled unless -unlabeled-value is specified. The value of <key> specifies
    what value in the imported file should be used as this label (these same key
    values are also used in the output file). The values of <red>, <green>,
    <blue> and <alpha> must be integers from 0 to 255, and will specify the
    color the label is drawn as (alpha of 255 means fully opaque, which is
    probably what you want).
    
    By default, it will create new label names with names like LABEL_5 for any
    values encountered that are not mentioned in the list file, specify
    -discard-others to instead set these values to the "unlabeled" key.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiLabelImportOutputs`).
    """
    cifti_label_import_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_LABEL_IMPORT_METADATA)
    params = execution.params(params)
    cargs = cifti_label_import_cargs(params, execution)
    ret = cifti_label_import_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_label_import(
    input_: InputPathType,
    label_list_file: str,
    output: str,
    file: str | None = None,
    value: int | None = None,
    drop_unused_labels: bool = False,
    discard_others: bool = False,
    runner: Runner | None = None,
) -> CiftiLabelImportOutputs:
    """
    MAKE A CIFTI LABEL FILE FROM A CIFTI FILE.
    
    Creates a cifti label file from a cifti file with label-like values. You may
    specify the empty string (use "") for <label-list-file>, which will be
    treated as if it is an empty file. The label list file must have the
    following format (2 lines per label):
    
    <labelname>
    <key> <red> <green> <blue> <alpha>
    ...
    
    Label names are specified on a separate line from their value and color, in
    order to let label names contain spaces. Whitespace is trimmed from both
    ends of the label name, but is kept if it is in the middle of a label. Do
    not specify the "unlabeled" key in the file, it is assumed that 0 means not
    labeled unless -unlabeled-value is specified. The value of <key> specifies
    what value in the imported file should be used as this label (these same key
    values are also used in the output file). The values of <red>, <green>,
    <blue> and <alpha> must be integers from 0 to 255, and will specify the
    color the label is drawn as (alpha of 255 means fully opaque, which is
    probably what you want).
    
    By default, it will create new label names with names like LABEL_5 for any
    values encountered that are not mentioned in the list file, specify
    -discard-others to instead set these values to the "unlabeled" key.
    
    Args:
        input_: the input cifti file.
        label_list_file: text file containing the values and names for labels.
        output: the output cifti label file.
        file: read label name hierarchy from a json file\
            \
            the input json file.
        value: set the value that will be interpreted as unlabeled\
            \
            the numeric value for unlabeled (default 0).
        drop_unused_labels: remove any unused label values from the label table.
        discard_others: set any values not mentioned in the label list to the\
            ??? label.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiLabelImportOutputs`).
    """
    params = cifti_label_import_params(
        input_=input_,
        label_list_file=label_list_file,
        output=output,
        file=file,
        value=value,
        drop_unused_labels=drop_unused_labels,
        discard_others=discard_others,
    )
    return cifti_label_import_execute(params, runner)


__all__ = [
    "CIFTI_LABEL_IMPORT_METADATA",
    "CiftiLabelImportOutputs",
    "CiftiLabelImportParamsDict",
    "CiftiLabelImportParamsDictTagged",
    "cifti_label_import",
    "cifti_label_import_execute",
    "cifti_label_import_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

NIFTI_INFORMATION_METADATA = Metadata(
    id="e9c11d632fc070fafb7efd4e280c2753fca13b1e.workbench",
    name="nifti-information",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_NiftiInformationPrintXmlParamsDictNoTag = typing.TypedDict('_NiftiInformationPrintXmlParamsDictNoTag', {
    "version": typing.NotRequired[str | None],
})
NiftiInformationPrintXmlParamsDictTagged = typing.TypedDict('NiftiInformationPrintXmlParamsDictTagged', {
    "@type": typing.Literal["print-xml"],
    "version": typing.NotRequired[str | None],
})
NiftiInformationPrintXmlParamsDict = _NiftiInformationPrintXmlParamsDictNoTag | NiftiInformationPrintXmlParamsDictTagged


_NiftiInformationParamsDictNoTag = typing.TypedDict('_NiftiInformationParamsDictNoTag', {
    "nifti-file": str,
    "print-xml": typing.NotRequired[NiftiInformationPrintXmlParamsDict | None],
    "allow-truncated": typing.NotRequired[bool | None],
    "print-matrix": bool,
})
NiftiInformationParamsDictTagged = typing.TypedDict('NiftiInformationParamsDictTagged', {
    "@type": typing.Literal["workbench/nifti-information"],
    "nifti-file": str,
    "print-xml": typing.NotRequired[NiftiInformationPrintXmlParamsDict | None],
    "allow-truncated": typing.NotRequired[bool | None],
    "print-matrix": bool,
})
NiftiInformationParamsDict = _NiftiInformationParamsDictNoTag | NiftiInformationParamsDictTagged


def nifti_information_print_xml(
    version: str | None = None,
) -> NiftiInformationPrintXmlParamsDictTagged:
    """
    Build parameters.
    
    Args:
        version: convert the XML to a specific CIFTI version (default is the\
            file's cifti version)\
            \
            the CIFTI version to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "print-xml",
    }
    if version is not None:
        params["version"] = version
    return params


def nifti_information_print_xml_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `NiftiInformationPrintXmlParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("version", None) is not None:
        if not isinstance(params["version"], str):
            raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", None))}` expected `str | None`')


def nifti_information_print_xml_cargs(
    params: NiftiInformationPrintXmlParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-print-xml")
    if params.get("version", None) is not None:
        cargs.extend([
            "-version",
            params.get("version", None)
        ])
    return cargs


class NiftiInformationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `NiftiInformationParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def nifti_information_params(
    nifti_file: str,
    print_xml: NiftiInformationPrintXmlParamsDict | None = None,
    allow_truncated: bool | None = None,
    print_matrix: bool = False,
) -> NiftiInformationParamsDictTagged:
    """
    Build parameters.
    
    Args:
        nifti_file: the nifti/cifti file to examine.
        print_xml: print the cifti XML (cifti only).
        allow_truncated: display the header contents\
            \
            print the header even if the data is truncated.
        print_matrix: output the values in the matrix (cifti only).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/nifti-information",
        "nifti-file": nifti_file,
        "print-matrix": print_matrix,
    }
    if print_xml is not None:
        params["print-xml"] = print_xml
    if allow_truncated is not None:
        params["allow-truncated"] = allow_truncated
    return params


def nifti_information_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `NiftiInformationParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("nifti-file", None) is None:
        raise StyxValidationError("`nifti-file` must not be None")
    if not isinstance(params["nifti-file"], str):
        raise StyxValidationError(f'`nifti-file` has the wrong type: Received `{type(params.get("nifti-file", None))}` expected `str`')
    if params.get("print-xml", None) is not None:
        nifti_information_print_xml_validate(params["print-xml"])
    if params.get("allow-truncated", None) is not None:
        if not isinstance(params["allow-truncated"], bool):
            raise StyxValidationError(f'`allow-truncated` has the wrong type: Received `{type(params.get("allow-truncated", None))}` expected `bool | None`')
    if params.get("print-matrix", False) is None:
        raise StyxValidationError("`print-matrix` must not be None")
    if not isinstance(params["print-matrix"], bool):
        raise StyxValidationError(f'`print-matrix` has the wrong type: Received `{type(params.get("print-matrix", False))}` expected `bool`')


def nifti_information_cargs(
    params: NiftiInformationParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-nifti-information"
    ])
    cargs.append(params.get("nifti-file", None))
    if params.get("print-xml", None) is not None:
        cargs.extend(nifti_information_print_xml_cargs(params.get("print-xml", None), execution))
    if params.get("allow-truncated", None) is not None:
        cargs.extend([
            "-print-header",
            "-allow-truncated"
        ])
    if params.get("print-matrix", False):
        cargs.append("-print-matrix")
    return cargs


def nifti_information_outputs(
    params: NiftiInformationParamsDict,
    execution: Execution,
) -> NiftiInformationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = NiftiInformationOutputs(
        root=execution.output_file("."),
    )
    return ret


def nifti_information_execute(
    params: NiftiInformationParamsDict,
    runner: Runner | None = None,
) -> NiftiInformationOutputs:
    """
    DISPLAY INFORMATION ABOUT A NIFTI/CIFTI FILE.
    
    You must specify at least one -print-* option.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `NiftiInformationOutputs`).
    """
    nifti_information_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(NIFTI_INFORMATION_METADATA)
    params = execution.params(params)
    cargs = nifti_information_cargs(params, execution)
    ret = nifti_information_outputs(params, execution)
    execution.run(cargs)
    return ret


def nifti_information(
    nifti_file: str,
    print_xml: NiftiInformationPrintXmlParamsDict | None = None,
    allow_truncated: bool | None = None,
    print_matrix: bool = False,
    runner: Runner | None = None,
) -> NiftiInformationOutputs:
    """
    DISPLAY INFORMATION ABOUT A NIFTI/CIFTI FILE.
    
    You must specify at least one -print-* option.
    
    Args:
        nifti_file: the nifti/cifti file to examine.
        print_xml: print the cifti XML (cifti only).
        allow_truncated: display the header contents\
            \
            print the header even if the data is truncated.
        print_matrix: output the values in the matrix (cifti only).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `NiftiInformationOutputs`).
    """
    params = nifti_information_params(
        nifti_file=nifti_file,
        print_xml=print_xml,
        allow_truncated=allow_truncated,
        print_matrix=print_matrix,
    )
    return nifti_information_execute(params, runner)


__all__ = [
    "NIFTI_INFORMATION_METADATA",
    "NiftiInformationOutputs",
    "NiftiInformationParamsDict",
    "NiftiInformationParamsDictTagged",
    "NiftiInformationPrintXmlParamsDict",
    "NiftiInformationPrintXmlParamsDictTagged",
    "nifti_information",
    "nifti_information_execute",
    "nifti_information_params",
    "nifti_information_print_xml",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_SMOOTHING_METADATA = Metadata(
    id="13acc3e6ded7de43dd47b9ee49249c55bebbc762.workbench",
    name="cifti-smoothing",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiSmoothingLeftSurfaceParamsDictNoTag = typing.TypedDict('_CiftiSmoothingLeftSurfaceParamsDictNoTag', {
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiSmoothingLeftSurfaceParamsDictTagged = typing.TypedDict('CiftiSmoothingLeftSurfaceParamsDictTagged', {
    "@type": typing.Literal["left-surface"],
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiSmoothingLeftSurfaceParamsDict = _CiftiSmoothingLeftSurfaceParamsDictNoTag | CiftiSmoothingLeftSurfaceParamsDictTagged


_CiftiSmoothingRightSurfaceParamsDictNoTag = typing.TypedDict('_CiftiSmoothingRightSurfaceParamsDictNoTag', {
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiSmoothingRightSurfaceParamsDictTagged = typing.TypedDict('CiftiSmoothingRightSurfaceParamsDictTagged', {
    "@type": typing.Literal["right-surface"],
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiSmoothingRightSurfaceParamsDict = _CiftiSmoothingRightSurfaceParamsDictNoTag | CiftiSmoothingRightSurfaceParamsDictTagged


_CiftiSmoothingCerebellumSurfaceParamsDictNoTag = typing.TypedDict('_CiftiSmoothingCerebellumSurfaceParamsDictNoTag', {
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiSmoothingCerebellumSurfaceParamsDictTagged = typing.TypedDict('CiftiSmoothingCerebellumSurfaceParamsDictTagged', {
    "@type": typing.Literal["cerebellum-surface"],
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiSmoothingCerebellumSurfaceParamsDict = _CiftiSmoothingCerebellumSurfaceParamsDictNoTag | CiftiSmoothingCerebellumSurfaceParamsDictTagged


_CiftiSmoothingSurfaceParamsDictNoTag = typing.TypedDict('_CiftiSmoothingSurfaceParamsDictNoTag', {
    "structure": str,
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiSmoothingSurfaceParamsDictTagged = typing.TypedDict('CiftiSmoothingSurfaceParamsDictTagged', {
    "@type": typing.Literal["surface"],
    "structure": str,
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiSmoothingSurfaceParamsDict = _CiftiSmoothingSurfaceParamsDictNoTag | CiftiSmoothingSurfaceParamsDictTagged


_CiftiSmoothingParamsDictNoTag = typing.TypedDict('_CiftiSmoothingParamsDictNoTag', {
    "cifti": InputPathType,
    "surface-kernel": float,
    "volume-kernel": float,
    "direction": str,
    "cifti-out": str,
    "left-surface": typing.NotRequired[CiftiSmoothingLeftSurfaceParamsDict | None],
    "right-surface": typing.NotRequired[CiftiSmoothingRightSurfaceParamsDict | None],
    "cerebellum-surface": typing.NotRequired[CiftiSmoothingCerebellumSurfaceParamsDict | None],
    "surface": typing.NotRequired[list[CiftiSmoothingSurfaceParamsDict] | None],
    "roi-cifti": typing.NotRequired[InputPathType | None],
    "merged-volume": bool,
    "fix-zeros-surface": bool,
    "fix-zeros-volume": bool,
    "fwhm": bool,
})
CiftiSmoothingParamsDictTagged = typing.TypedDict('CiftiSmoothingParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-smoothing"],
    "cifti": InputPathType,
    "surface-kernel": float,
    "volume-kernel": float,
    "direction": str,
    "cifti-out": str,
    "left-surface": typing.NotRequired[CiftiSmoothingLeftSurfaceParamsDict | None],
    "right-surface": typing.NotRequired[CiftiSmoothingRightSurfaceParamsDict | None],
    "cerebellum-surface": typing.NotRequired[CiftiSmoothingCerebellumSurfaceParamsDict | None],
    "surface": typing.NotRequired[list[CiftiSmoothingSurfaceParamsDict] | None],
    "roi-cifti": typing.NotRequired[InputPathType | None],
    "merged-volume": bool,
    "fix-zeros-surface": bool,
    "fix-zeros-volume": bool,
    "fwhm": bool,
})
CiftiSmoothingParamsDict = _CiftiSmoothingParamsDictNoTag | CiftiSmoothingParamsDictTagged


def cifti_smoothing_left_surface(
    surface: InputPathType,
    area_metric: InputPathType | None = None,
) -> CiftiSmoothingLeftSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the left surface file.
        area_metric: vertex areas to use instead of computing them from the\
            left surface\
            \
            the corrected vertex areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left-surface",
        "surface": surface,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def cifti_smoothing_left_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSmoothingLeftSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')


def cifti_smoothing_left_surface_cargs(
    params: CiftiSmoothingLeftSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-left-surface",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-left-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    return cargs


def cifti_smoothing_right_surface(
    surface: InputPathType,
    area_metric: InputPathType | None = None,
) -> CiftiSmoothingRightSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the right surface file.
        area_metric: vertex areas to use instead of computing them from the\
            right surface\
            \
            the corrected vertex areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right-surface",
        "surface": surface,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def cifti_smoothing_right_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSmoothingRightSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')


def cifti_smoothing_right_surface_cargs(
    params: CiftiSmoothingRightSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-right-surface",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-right-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    return cargs


def cifti_smoothing_cerebellum_surface(
    surface: InputPathType,
    area_metric: InputPathType | None = None,
) -> CiftiSmoothingCerebellumSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the cerebellum surface file.
        area_metric: vertex areas to use instead of computing them from the\
            cerebellum surface\
            \
            the corrected vertex areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum-surface",
        "surface": surface,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def cifti_smoothing_cerebellum_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSmoothingCerebellumSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')


def cifti_smoothing_cerebellum_surface_cargs(
    params: CiftiSmoothingCerebellumSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cerebellum-surface",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-cerebellum-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    return cargs


def cifti_smoothing_surface(
    structure: str,
    surface: InputPathType,
    area_metric: InputPathType | None = None,
) -> CiftiSmoothingSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the surface structure name.
        surface: the surface file.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "surface",
        "structure": structure,
        "surface": surface,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def cifti_smoothing_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSmoothingSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')


def cifti_smoothing_surface_cargs(
    params: CiftiSmoothingSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-surface",
        params.get("structure", None),
        execution.input_file(params.get("surface", None))
    ])
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    return cargs


class CiftiSmoothingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiSmoothingParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti"""


def cifti_smoothing_params(
    cifti: InputPathType,
    surface_kernel: float,
    volume_kernel: float,
    direction: str,
    cifti_out: str,
    left_surface: CiftiSmoothingLeftSurfaceParamsDict | None = None,
    right_surface: CiftiSmoothingRightSurfaceParamsDict | None = None,
    cerebellum_surface: CiftiSmoothingCerebellumSurfaceParamsDict | None = None,
    surface: list[CiftiSmoothingSurfaceParamsDict] | None = None,
    roi_cifti: InputPathType | None = None,
    merged_volume: bool = False,
    fix_zeros_surface: bool = False,
    fix_zeros_volume: bool = False,
    fwhm: bool = False,
) -> CiftiSmoothingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti: the input cifti.
        surface_kernel: the size of the gaussian surface smoothing kernel in\
            mm, as sigma by default.
        volume_kernel: the size of the gaussian volume smoothing kernel in mm,\
            as sigma by default.
        direction: which dimension to smooth along, ROW or COLUMN.
        cifti_out: the output cifti.
        left_surface: specify the left cortical surface to use.
        right_surface: specify the right cortical surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        surface: specify a surface by structure name.
        roi_cifti: smooth only within regions of interest\
            \
            the regions to smooth within, as a cifti file.
        merged_volume: smooth across subcortical structure boundaries.
        fix_zeros_surface: treat values of zero on the surface as missing data.
        fix_zeros_volume: treat values of zero in the volume as missing data.
        fwhm: kernel sizes are FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-smoothing",
        "cifti": cifti,
        "surface-kernel": surface_kernel,
        "volume-kernel": volume_kernel,
        "direction": direction,
        "cifti-out": cifti_out,
        "merged-volume": merged_volume,
        "fix-zeros-surface": fix_zeros_surface,
        "fix-zeros-volume": fix_zeros_volume,
        "fwhm": fwhm,
    }
    if left_surface is not None:
        params["left-surface"] = left_surface
    if right_surface is not None:
        params["right-surface"] = right_surface
    if cerebellum_surface is not None:
        params["cerebellum-surface"] = cerebellum_surface
    if surface is not None:
        params["surface"] = surface
    if roi_cifti is not None:
        params["roi-cifti"] = roi_cifti
    return params


def cifti_smoothing_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSmoothingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti", None) is None:
        raise StyxValidationError("`cifti` must not be None")
    if not isinstance(params["cifti"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `InputPathType`')
    if params.get("surface-kernel", None) is None:
        raise StyxValidationError("`surface-kernel` must not be None")
    if not isinstance(params["surface-kernel"], (float, int)):
        raise StyxValidationError(f'`surface-kernel` has the wrong type: Received `{type(params.get("surface-kernel", None))}` expected `float`')
    if params.get("volume-kernel", None) is None:
        raise StyxValidationError("`volume-kernel` must not be None")
    if not isinstance(params["volume-kernel"], (float, int)):
        raise StyxValidationError(f'`volume-kernel` has the wrong type: Received `{type(params.get("volume-kernel", None))}` expected `float`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("left-surface", None) is not None:
        cifti_smoothing_left_surface_validate(params["left-surface"])
    if params.get("right-surface", None) is not None:
        cifti_smoothing_right_surface_validate(params["right-surface"])
    if params.get("cerebellum-surface", None) is not None:
        cifti_smoothing_cerebellum_surface_validate(params["cerebellum-surface"])
    if params.get("surface", None) is not None:
        if not isinstance(params["surface"], list):
            raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `list[CiftiSmoothingSurfaceParamsDict] | None`')
        for e in params["surface"]:
            cifti_smoothing_surface_validate(e)
    if params.get("roi-cifti", None) is not None:
        if not isinstance(params["roi-cifti"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-cifti` has the wrong type: Received `{type(params.get("roi-cifti", None))}` expected `InputPathType | None`')
    if params.get("merged-volume", False) is None:
        raise StyxValidationError("`merged-volume` must not be None")
    if not isinstance(params["merged-volume"], bool):
        raise StyxValidationError(f'`merged-volume` has the wrong type: Received `{type(params.get("merged-volume", False))}` expected `bool`')
    if params.get("fix-zeros-surface", False) is None:
        raise StyxValidationError("`fix-zeros-surface` must not be None")
    if not isinstance(params["fix-zeros-surface"], bool):
        raise StyxValidationError(f'`fix-zeros-surface` has the wrong type: Received `{type(params.get("fix-zeros-surface", False))}` expected `bool`')
    if params.get("fix-zeros-volume", False) is None:
        raise StyxValidationError("`fix-zeros-volume` must not be None")
    if not isinstance(params["fix-zeros-volume"], bool):
        raise StyxValidationError(f'`fix-zeros-volume` has the wrong type: Received `{type(params.get("fix-zeros-volume", False))}` expected `bool`')
    if params.get("fwhm", False) is None:
        raise StyxValidationError("`fwhm` must not be None")
    if not isinstance(params["fwhm"], bool):
        raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", False))}` expected `bool`')


def cifti_smoothing_cargs(
    params: CiftiSmoothingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-smoothing"
    ])
    cargs.append(execution.input_file(params.get("cifti", None)))
    cargs.append(str(params.get("surface-kernel", None)))
    cargs.append(str(params.get("volume-kernel", None)))
    cargs.append(params.get("direction", None))
    cargs.append(params.get("cifti-out", None))
    if params.get("left-surface", None) is not None or params.get("right-surface", None) is not None or params.get("cerebellum-surface", None) is not None or params.get("surface", None) is not None:
        cargs.extend([
            *(cifti_smoothing_left_surface_cargs(params.get("left-surface", None), execution) if (params.get("left-surface", None) is not None) else []),
            *(cifti_smoothing_right_surface_cargs(params.get("right-surface", None), execution) if (params.get("right-surface", None) is not None) else []),
            *(cifti_smoothing_cerebellum_surface_cargs(params.get("cerebellum-surface", None), execution) if (params.get("cerebellum-surface", None) is not None) else []),
            *([a for c in [cifti_smoothing_surface_cargs(s, execution) for s in params.get("surface", None)] for a in c] if (params.get("surface", None) is not None) else [])
        ])
    if params.get("roi-cifti", None) is not None:
        cargs.extend([
            "-cifti-roi",
            execution.input_file(params.get("roi-cifti", None))
        ])
    if params.get("merged-volume", False):
        cargs.append("-merged-volume")
    if params.get("fix-zeros-surface", False):
        cargs.append("-fix-zeros-surface")
    if params.get("fix-zeros-volume", False):
        cargs.append("-fix-zeros-volume")
    if params.get("fwhm", False):
        cargs.append("-fwhm")
    return cargs


def cifti_smoothing_outputs(
    params: CiftiSmoothingParamsDict,
    execution: Execution,
) -> CiftiSmoothingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiSmoothingOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_smoothing_execute(
    params: CiftiSmoothingParamsDict,
    runner: Runner | None = None,
) -> CiftiSmoothingOutputs:
    """
    SMOOTH A CIFTI FILE.
    
    The input cifti file must have a brain models mapping on the chosen
    dimension, columns for .dtseries, and either for .dconn. By default, data in
    different structures is smoothed independently (i.e., "parcel constrained"
    smoothing), so volume structures that touch do not smooth across this
    boundary. Specify -merged-volume to ignore these boundaries. Surface
    smoothing uses the GEO_GAUSS_AREA smoothing method.
    
    The -*-corrected-areas options are intended for when it is unavoidable to
    smooth on group average surfaces, it is only an approximate correction for
    the reduction of structure in a group average surface. It is better to
    smooth the data on individuals before averaging, when feasible.
    
    The -fix-zeros-* options will treat values of zero as lack of data, and not
    use that value when generating the smoothed values, but will fill zeros with
    extrapolated values. The ROI should have a brain models mapping along
    columns, exactly matching the mapping of the chosen direction in the input
    file. Data outside the ROI is ignored.
    
    The <structure> argument of -surface must be one of the following strings:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiSmoothingOutputs`).
    """
    cifti_smoothing_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_SMOOTHING_METADATA)
    params = execution.params(params)
    cargs = cifti_smoothing_cargs(params, execution)
    ret = cifti_smoothing_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_smoothing(
    cifti: InputPathType,
    surface_kernel: float,
    volume_kernel: float,
    direction: str,
    cifti_out: str,
    left_surface: CiftiSmoothingLeftSurfaceParamsDict | None = None,
    right_surface: CiftiSmoothingRightSurfaceParamsDict | None = None,
    cerebellum_surface: CiftiSmoothingCerebellumSurfaceParamsDict | None = None,
    surface: list[CiftiSmoothingSurfaceParamsDict] | None = None,
    roi_cifti: InputPathType | None = None,
    merged_volume: bool = False,
    fix_zeros_surface: bool = False,
    fix_zeros_volume: bool = False,
    fwhm: bool = False,
    runner: Runner | None = None,
) -> CiftiSmoothingOutputs:
    """
    SMOOTH A CIFTI FILE.
    
    The input cifti file must have a brain models mapping on the chosen
    dimension, columns for .dtseries, and either for .dconn. By default, data in
    different structures is smoothed independently (i.e., "parcel constrained"
    smoothing), so volume structures that touch do not smooth across this
    boundary. Specify -merged-volume to ignore these boundaries. Surface
    smoothing uses the GEO_GAUSS_AREA smoothing method.
    
    The -*-corrected-areas options are intended for when it is unavoidable to
    smooth on group average surfaces, it is only an approximate correction for
    the reduction of structure in a group average surface. It is better to
    smooth the data on individuals before averaging, when feasible.
    
    The -fix-zeros-* options will treat values of zero as lack of data, and not
    use that value when generating the smoothed values, but will fill zeros with
    extrapolated values. The ROI should have a brain models mapping along
    columns, exactly matching the mapping of the chosen direction in the input
    file. Data outside the ROI is ignored.
    
    The <structure> argument of -surface must be one of the following strings:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        cifti: the input cifti.
        surface_kernel: the size of the gaussian surface smoothing kernel in\
            mm, as sigma by default.
        volume_kernel: the size of the gaussian volume smoothing kernel in mm,\
            as sigma by default.
        direction: which dimension to smooth along, ROW or COLUMN.
        cifti_out: the output cifti.
        left_surface: specify the left cortical surface to use.
        right_surface: specify the right cortical surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        surface: specify a surface by structure name.
        roi_cifti: smooth only within regions of interest\
            \
            the regions to smooth within, as a cifti file.
        merged_volume: smooth across subcortical structure boundaries.
        fix_zeros_surface: treat values of zero on the surface as missing data.
        fix_zeros_volume: treat values of zero in the volume as missing data.
        fwhm: kernel sizes are FWHM, not sigma.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiSmoothingOutputs`).
    """
    params = cifti_smoothing_params(
        cifti=cifti,
        surface_kernel=surface_kernel,
        volume_kernel=volume_kernel,
        direction=direction,
        cifti_out=cifti_out,
        left_surface=left_surface,
        right_surface=right_surface,
        cerebellum_surface=cerebellum_surface,
        surface=surface,
        roi_cifti=roi_cifti,
        merged_volume=merged_volume,
        fix_zeros_surface=fix_zeros_surface,
        fix_zeros_volume=fix_zeros_volume,
        fwhm=fwhm,
    )
    return cifti_smoothing_execute(params, runner)


__all__ = [
    "CIFTI_SMOOTHING_METADATA",
    "CiftiSmoothingCerebellumSurfaceParamsDict",
    "CiftiSmoothingCerebellumSurfaceParamsDictTagged",
    "CiftiSmoothingLeftSurfaceParamsDict",
    "CiftiSmoothingLeftSurfaceParamsDictTagged",
    "CiftiSmoothingOutputs",
    "CiftiSmoothingParamsDict",
    "CiftiSmoothingParamsDictTagged",
    "CiftiSmoothingRightSurfaceParamsDict",
    "CiftiSmoothingRightSurfaceParamsDictTagged",
    "CiftiSmoothingSurfaceParamsDict",
    "CiftiSmoothingSurfaceParamsDictTagged",
    "cifti_smoothing",
    "cifti_smoothing_cerebellum_surface",
    "cifti_smoothing_execute",
    "cifti_smoothing_left_surface",
    "cifti_smoothing_params",
    "cifti_smoothing_right_surface",
    "cifti_smoothing_surface",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CONVERT_MATRIX4_TO_MATRIX2_METADATA = Metadata(
    id="6b1c39b77ea63d80e9eaa7d762e583b3aa47688b.workbench",
    name="convert-matrix4-to-matrix2",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_ConvertMatrix4ToMatrix2DistancesParamsDictNoTag = typing.TypedDict('_ConvertMatrix4ToMatrix2DistancesParamsDictNoTag', {
    "distance-out": str,
})
ConvertMatrix4ToMatrix2DistancesParamsDictTagged = typing.TypedDict('ConvertMatrix4ToMatrix2DistancesParamsDictTagged', {
    "@type": typing.Literal["distances"],
    "distance-out": str,
})
ConvertMatrix4ToMatrix2DistancesParamsDict = _ConvertMatrix4ToMatrix2DistancesParamsDictNoTag | ConvertMatrix4ToMatrix2DistancesParamsDictTagged


_ConvertMatrix4ToMatrix2IndividualFibersParamsDictNoTag = typing.TypedDict('_ConvertMatrix4ToMatrix2IndividualFibersParamsDictNoTag', {
    "fiber-1": str,
    "fiber-2": str,
    "fiber-3": str,
})
ConvertMatrix4ToMatrix2IndividualFibersParamsDictTagged = typing.TypedDict('ConvertMatrix4ToMatrix2IndividualFibersParamsDictTagged', {
    "@type": typing.Literal["individual-fibers"],
    "fiber-1": str,
    "fiber-2": str,
    "fiber-3": str,
})
ConvertMatrix4ToMatrix2IndividualFibersParamsDict = _ConvertMatrix4ToMatrix2IndividualFibersParamsDictNoTag | ConvertMatrix4ToMatrix2IndividualFibersParamsDictTagged


_ConvertMatrix4ToMatrix2ParamsDictNoTag = typing.TypedDict('_ConvertMatrix4ToMatrix2ParamsDictNoTag', {
    "matrix4-wbsparse": str,
    "counts-out": str,
    "distances": typing.NotRequired[ConvertMatrix4ToMatrix2DistancesParamsDict | None],
    "individual-fibers": typing.NotRequired[ConvertMatrix4ToMatrix2IndividualFibersParamsDict | None],
})
ConvertMatrix4ToMatrix2ParamsDictTagged = typing.TypedDict('ConvertMatrix4ToMatrix2ParamsDictTagged', {
    "@type": typing.Literal["workbench/convert-matrix4-to-matrix2"],
    "matrix4-wbsparse": str,
    "counts-out": str,
    "distances": typing.NotRequired[ConvertMatrix4ToMatrix2DistancesParamsDict | None],
    "individual-fibers": typing.NotRequired[ConvertMatrix4ToMatrix2IndividualFibersParamsDict | None],
})
ConvertMatrix4ToMatrix2ParamsDict = _ConvertMatrix4ToMatrix2ParamsDictNoTag | ConvertMatrix4ToMatrix2ParamsDictTagged


class ConvertMatrix4ToMatrix2DistancesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ConvertMatrix4ToMatrix2DistancesParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    distance_out: OutputPathType
    """the distances, as a cifti file"""


def convert_matrix4_to_matrix2_distances(
    distance_out: str,
) -> ConvertMatrix4ToMatrix2DistancesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        distance_out: the distances, as a cifti file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "distances",
        "distance-out": distance_out,
    }
    return params


def convert_matrix4_to_matrix2_distances_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertMatrix4ToMatrix2DistancesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("distance-out", None) is None:
        raise StyxValidationError("`distance-out` must not be None")
    if not isinstance(params["distance-out"], str):
        raise StyxValidationError(f'`distance-out` has the wrong type: Received `{type(params.get("distance-out", None))}` expected `str`')


def convert_matrix4_to_matrix2_distances_cargs(
    params: ConvertMatrix4ToMatrix2DistancesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-distances",
        params.get("distance-out", None)
    ])
    return cargs


def convert_matrix4_to_matrix2_distances_outputs(
    params: ConvertMatrix4ToMatrix2DistancesParamsDict,
    execution: Execution,
) -> ConvertMatrix4ToMatrix2DistancesOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ConvertMatrix4ToMatrix2DistancesOutputs(
        root=execution.output_file("."),
        distance_out=execution.output_file(params.get("distance-out", None)),
    )
    return ret


class ConvertMatrix4ToMatrix2IndividualFibersOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ConvertMatrix4ToMatrix2IndividualFibersParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    fiber_1: OutputPathType
    """output file for first fiber"""
    fiber_2: OutputPathType
    """output file for second fiber"""
    fiber_3: OutputPathType
    """output file for third fiber"""


def convert_matrix4_to_matrix2_individual_fibers(
    fiber_1: str,
    fiber_2: str,
    fiber_3: str,
) -> ConvertMatrix4ToMatrix2IndividualFibersParamsDictTagged:
    """
    Build parameters.
    
    Args:
        fiber_1: output file for first fiber.
        fiber_2: output file for second fiber.
        fiber_3: output file for third fiber.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "individual-fibers",
        "fiber-1": fiber_1,
        "fiber-2": fiber_2,
        "fiber-3": fiber_3,
    }
    return params


def convert_matrix4_to_matrix2_individual_fibers_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertMatrix4ToMatrix2IndividualFibersParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("fiber-1", None) is None:
        raise StyxValidationError("`fiber-1` must not be None")
    if not isinstance(params["fiber-1"], str):
        raise StyxValidationError(f'`fiber-1` has the wrong type: Received `{type(params.get("fiber-1", None))}` expected `str`')
    if params.get("fiber-2", None) is None:
        raise StyxValidationError("`fiber-2` must not be None")
    if not isinstance(params["fiber-2"], str):
        raise StyxValidationError(f'`fiber-2` has the wrong type: Received `{type(params.get("fiber-2", None))}` expected `str`')
    if params.get("fiber-3", None) is None:
        raise StyxValidationError("`fiber-3` must not be None")
    if not isinstance(params["fiber-3"], str):
        raise StyxValidationError(f'`fiber-3` has the wrong type: Received `{type(params.get("fiber-3", None))}` expected `str`')


def convert_matrix4_to_matrix2_individual_fibers_cargs(
    params: ConvertMatrix4ToMatrix2IndividualFibersParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-individual-fibers",
        params.get("fiber-1", None),
        params.get("fiber-2", None),
        params.get("fiber-3", None)
    ])
    return cargs


def convert_matrix4_to_matrix2_individual_fibers_outputs(
    params: ConvertMatrix4ToMatrix2IndividualFibersParamsDict,
    execution: Execution,
) -> ConvertMatrix4ToMatrix2IndividualFibersOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ConvertMatrix4ToMatrix2IndividualFibersOutputs(
        root=execution.output_file("."),
        fiber_1=execution.output_file(params.get("fiber-1", None)),
        fiber_2=execution.output_file(params.get("fiber-2", None)),
        fiber_3=execution.output_file(params.get("fiber-3", None)),
    )
    return ret


class ConvertMatrix4ToMatrix2Outputs(typing.NamedTuple):
    """
    Output object returned when calling `ConvertMatrix4ToMatrix2ParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    counts_out: OutputPathType
    """the total fiber counts, as a cifti file"""
    distances: ConvertMatrix4ToMatrix2DistancesOutputs | None
    """Outputs from `convert_matrix4_to_matrix2_distances_outputs`."""
    individual_fibers: ConvertMatrix4ToMatrix2IndividualFibersOutputs | None
    """Outputs from `convert_matrix4_to_matrix2_individual_fibers_outputs`."""


def convert_matrix4_to_matrix2_params(
    matrix4_wbsparse: str,
    counts_out: str,
    distances: ConvertMatrix4ToMatrix2DistancesParamsDict | None = None,
    individual_fibers: ConvertMatrix4ToMatrix2IndividualFibersParamsDict | None = None,
) -> ConvertMatrix4ToMatrix2ParamsDictTagged:
    """
    Build parameters.
    
    Args:
        matrix4_wbsparse: a wbsparse matrix4 file.
        counts_out: the total fiber counts, as a cifti file.
        distances: output average trajectory distance.
        individual_fibers: output files for each fiber direction.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/convert-matrix4-to-matrix2",
        "matrix4-wbsparse": matrix4_wbsparse,
        "counts-out": counts_out,
    }
    if distances is not None:
        params["distances"] = distances
    if individual_fibers is not None:
        params["individual-fibers"] = individual_fibers
    return params


def convert_matrix4_to_matrix2_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertMatrix4ToMatrix2ParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("matrix4-wbsparse", None) is None:
        raise StyxValidationError("`matrix4-wbsparse` must not be None")
    if not isinstance(params["matrix4-wbsparse"], str):
        raise StyxValidationError(f'`matrix4-wbsparse` has the wrong type: Received `{type(params.get("matrix4-wbsparse", None))}` expected `str`')
    if params.get("counts-out", None) is None:
        raise StyxValidationError("`counts-out` must not be None")
    if not isinstance(params["counts-out"], str):
        raise StyxValidationError(f'`counts-out` has the wrong type: Received `{type(params.get("counts-out", None))}` expected `str`')
    if params.get("distances", None) is not None:
        convert_matrix4_to_matrix2_distances_validate(params["distances"])
    if params.get("individual-fibers", None) is not None:
        convert_matrix4_to_matrix2_individual_fibers_validate(params["individual-fibers"])


def convert_matrix4_to_matrix2_cargs(
    params: ConvertMatrix4ToMatrix2ParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-convert-matrix4-to-matrix2"
    ])
    cargs.append(params.get("matrix4-wbsparse", None))
    cargs.append(params.get("counts-out", None))
    if params.get("distances", None) is not None or params.get("individual-fibers", None) is not None:
        cargs.extend([
            *(convert_matrix4_to_matrix2_distances_cargs(params.get("distances", None), execution) if (params.get("distances", None) is not None) else []),
            *(convert_matrix4_to_matrix2_individual_fibers_cargs(params.get("individual-fibers", None), execution) if (params.get("individual-fibers", None) is not None) else [])
        ])
    return cargs


def convert_matrix4_to_matrix2_outputs(
    params: ConvertMatrix4ToMatrix2ParamsDict,
    execution: Execution,
) -> ConvertMatrix4ToMatrix2Outputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ConvertMatrix4ToMatrix2Outputs(
        root=execution.output_file("."),
        counts_out=execution.output_file(params.get("counts-out", None)),
        distances=convert_matrix4_to_matrix2_distances_outputs(params.get("distances"), execution) if params.get("distances") else None,
        individual_fibers=convert_matrix4_to_matrix2_individual_fibers_outputs(params.get("individual-fibers"), execution) if params.get("individual-fibers") else None,
    )
    return ret


def convert_matrix4_to_matrix2_execute(
    params: ConvertMatrix4ToMatrix2ParamsDict,
    runner: Runner | None = None,
) -> ConvertMatrix4ToMatrix2Outputs:
    """
    GENERATES A MATRIX2 CIFTI FROM MATRIX4 WBSPARSE.
    
    This command makes a cifti file from the fiber counts in a matrix4 wbsparse
    file, and optionally a second cifti file from the distances. Note that while
    the total count is stored exactly, the per-fiber counts are stored as
    approximate fractions, so the output of -individual-fibers will contain
    nonintegers.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertMatrix4ToMatrix2Outputs`).
    """
    convert_matrix4_to_matrix2_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVERT_MATRIX4_TO_MATRIX2_METADATA)
    params = execution.params(params)
    cargs = convert_matrix4_to_matrix2_cargs(params, execution)
    ret = convert_matrix4_to_matrix2_outputs(params, execution)
    execution.run(cargs)
    return ret


def convert_matrix4_to_matrix2(
    matrix4_wbsparse: str,
    counts_out: str,
    distances: ConvertMatrix4ToMatrix2DistancesParamsDict | None = None,
    individual_fibers: ConvertMatrix4ToMatrix2IndividualFibersParamsDict | None = None,
    runner: Runner | None = None,
) -> ConvertMatrix4ToMatrix2Outputs:
    """
    GENERATES A MATRIX2 CIFTI FROM MATRIX4 WBSPARSE.
    
    This command makes a cifti file from the fiber counts in a matrix4 wbsparse
    file, and optionally a second cifti file from the distances. Note that while
    the total count is stored exactly, the per-fiber counts are stored as
    approximate fractions, so the output of -individual-fibers will contain
    nonintegers.
    
    Args:
        matrix4_wbsparse: a wbsparse matrix4 file.
        counts_out: the total fiber counts, as a cifti file.
        distances: output average trajectory distance.
        individual_fibers: output files for each fiber direction.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertMatrix4ToMatrix2Outputs`).
    """
    params = convert_matrix4_to_matrix2_params(
        matrix4_wbsparse=matrix4_wbsparse,
        counts_out=counts_out,
        distances=distances,
        individual_fibers=individual_fibers,
    )
    return convert_matrix4_to_matrix2_execute(params, runner)


__all__ = [
    "CONVERT_MATRIX4_TO_MATRIX2_METADATA",
    "ConvertMatrix4ToMatrix2DistancesOutputs",
    "ConvertMatrix4ToMatrix2DistancesParamsDict",
    "ConvertMatrix4ToMatrix2DistancesParamsDictTagged",
    "ConvertMatrix4ToMatrix2IndividualFibersOutputs",
    "ConvertMatrix4ToMatrix2IndividualFibersParamsDict",
    "ConvertMatrix4ToMatrix2IndividualFibersParamsDictTagged",
    "ConvertMatrix4ToMatrix2Outputs",
    "ConvertMatrix4ToMatrix2ParamsDict",
    "ConvertMatrix4ToMatrix2ParamsDictTagged",
    "convert_matrix4_to_matrix2",
    "convert_matrix4_to_matrix2_distances",
    "convert_matrix4_to_matrix2_execute",
    "convert_matrix4_to_matrix2_individual_fibers",
    "convert_matrix4_to_matrix2_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SET_MAP_NAMES_METADATA = Metadata(
    id="05c23caac5c21808cadb7f27b12f6fd3f1d3c1f7.workbench",
    name="set-map-names",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SetMapNamesMapParamsDictNoTag = typing.TypedDict('_SetMapNamesMapParamsDictNoTag', {
    "index": int,
    "new-name": str,
})
SetMapNamesMapParamsDictTagged = typing.TypedDict('SetMapNamesMapParamsDictTagged', {
    "@type": typing.Literal["map"],
    "index": int,
    "new-name": str,
})
SetMapNamesMapParamsDict = _SetMapNamesMapParamsDictNoTag | SetMapNamesMapParamsDictTagged


_SetMapNamesParamsDictNoTag = typing.TypedDict('_SetMapNamesParamsDictNoTag', {
    "data-file": str,
    "map": typing.NotRequired[list[SetMapNamesMapParamsDict] | None],
    "file": typing.NotRequired[str | None],
    "file": typing.NotRequired[str | None],
})
SetMapNamesParamsDictTagged = typing.TypedDict('SetMapNamesParamsDictTagged', {
    "@type": typing.Literal["workbench/set-map-names"],
    "data-file": str,
    "map": typing.NotRequired[list[SetMapNamesMapParamsDict] | None],
    "file": typing.NotRequired[str | None],
    "file": typing.NotRequired[str | None],
})
SetMapNamesParamsDict = _SetMapNamesParamsDictNoTag | SetMapNamesParamsDictTagged


def set_map_names_map(
    index: int,
    new_name: str,
) -> SetMapNamesMapParamsDictTagged:
    """
    Build parameters.
    
    Args:
        index: the map index to change the name of.
        new_name: the name to set for the map.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "map",
        "index": index,
        "new-name": new_name,
    }
    return params


def set_map_names_map_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SetMapNamesMapParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("index", None) is None:
        raise StyxValidationError("`index` must not be None")
    if not isinstance(params["index"], int):
        raise StyxValidationError(f'`index` has the wrong type: Received `{type(params.get("index", None))}` expected `int`')
    if params.get("new-name", None) is None:
        raise StyxValidationError("`new-name` must not be None")
    if not isinstance(params["new-name"], str):
        raise StyxValidationError(f'`new-name` has the wrong type: Received `{type(params.get("new-name", None))}` expected `str`')


def set_map_names_map_cargs(
    params: SetMapNamesMapParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-map",
        str(params.get("index", None)),
        params.get("new-name", None)
    ])
    return cargs


class SetMapNamesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SetMapNamesParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def set_map_names_params(
    data_file: str,
    map_: list[SetMapNamesMapParamsDict] | None = None,
    file: str | None = None,
    file_: str | None = None,
) -> SetMapNamesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        data_file: the file to set the map names of.
        map_: specify a map to set the name of.
        file: use the map names from another data file\
            \
            a data file with the same number of maps.
        file_: use a text file to replace all map names\
            \
            text file containing map names, one per line.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/set-map-names",
        "data-file": data_file,
    }
    if map_ is not None:
        params["map"] = map_
    if file is not None:
        params["file"] = file
    if file_ is not None:
        params["file"] = file_
    return params


def set_map_names_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SetMapNamesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("data-file", None) is None:
        raise StyxValidationError("`data-file` must not be None")
    if not isinstance(params["data-file"], str):
        raise StyxValidationError(f'`data-file` has the wrong type: Received `{type(params.get("data-file", None))}` expected `str`')
    if params.get("map", None) is not None:
        if not isinstance(params["map"], list):
            raise StyxValidationError(f'`map` has the wrong type: Received `{type(params.get("map", None))}` expected `list[SetMapNamesMapParamsDict] | None`')
        for e in params["map"]:
            set_map_names_map_validate(e)
    if params.get("file", None) is not None:
        if not isinstance(params["file"], str):
            raise StyxValidationError(f'`file` has the wrong type: Received `{type(params.get("file", None))}` expected `str | None`')
    if params.get("file", None) is not None:
        if not isinstance(params["file"], str):
            raise StyxValidationError(f'`file` has the wrong type: Received `{type(params.get("file", None))}` expected `str | None`')


def set_map_names_cargs(
    params: SetMapNamesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-set-map-names"
    ])
    cargs.append(params.get("data-file", None))
    if params.get("map", None) is not None:
        cargs.extend([a for c in [set_map_names_map_cargs(s, execution) for s in params.get("map", None)] for a in c])
    if params.get("file", None) is not None:
        cargs.extend([
            "-from-data-file",
            params.get("file", None)
        ])
    if params.get("file", None) is not None:
        cargs.extend([
            "-name-file",
            params.get("file", None)
        ])
    return cargs


def set_map_names_outputs(
    params: SetMapNamesParamsDict,
    execution: Execution,
) -> SetMapNamesOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SetMapNamesOutputs(
        root=execution.output_file("."),
    )
    return ret


def set_map_names_execute(
    params: SetMapNamesParamsDict,
    runner: Runner | None = None,
) -> SetMapNamesOutputs:
    """
    SET THE NAME OF ONE OR MORE MAPS IN A FILE.
    
    Sets the name of one or more maps for metric, shape, label, volume, cifti
    scalar or cifti label files. You must specify either -name-file, or
    -from-data-file, or at least one -map option. The three option types are
    mutually exclusive.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SetMapNamesOutputs`).
    """
    set_map_names_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SET_MAP_NAMES_METADATA)
    params = execution.params(params)
    cargs = set_map_names_cargs(params, execution)
    ret = set_map_names_outputs(params, execution)
    execution.run(cargs)
    return ret


def set_map_names(
    data_file: str,
    map_: list[SetMapNamesMapParamsDict] | None = None,
    file: str | None = None,
    file_: str | None = None,
    runner: Runner | None = None,
) -> SetMapNamesOutputs:
    """
    SET THE NAME OF ONE OR MORE MAPS IN A FILE.
    
    Sets the name of one or more maps for metric, shape, label, volume, cifti
    scalar or cifti label files. You must specify either -name-file, or
    -from-data-file, or at least one -map option. The three option types are
    mutually exclusive.
    
    Args:
        data_file: the file to set the map names of.
        map_: specify a map to set the name of.
        file: use the map names from another data file\
            \
            a data file with the same number of maps.
        file_: use a text file to replace all map names\
            \
            text file containing map names, one per line.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SetMapNamesOutputs`).
    """
    params = set_map_names_params(
        data_file=data_file,
        map_=map_,
        file=file,
        file_=file_,
    )
    return set_map_names_execute(params, runner)


__all__ = [
    "SET_MAP_NAMES_METADATA",
    "SetMapNamesMapParamsDict",
    "SetMapNamesMapParamsDictTagged",
    "SetMapNamesOutputs",
    "SetMapNamesParamsDict",
    "SetMapNamesParamsDictTagged",
    "set_map_names",
    "set_map_names_execute",
    "set_map_names_map",
    "set_map_names_params",
]
